
----------------------------------------------------------------------
--
--  CBP 01-12-14 :
--
--  ISE_LDA_tb	 TestBench for Control circuit LDA op-code
--
--
-----------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.numeric_std.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY LDA_tb IS
END LDA_tb;

ARCHITECTURE behavior OF LDA_tb IS 

	COMPONENT LDA is
    Port ( 
    reset : in std_logic;
    mio : out std_logic;
    mwe : out std_logic;

	-- registers
	IR : in std_logic_vector(15 downto 0); -- instruction register
	IP : in std_logic_vector(15 downto 0); -- instruction pointer
	ACC : inout std_logic_vector(15 downto 0);	-- accumulator
	zeroACC : inout std_logic;
	
    -- control unit
	cState : in std_logic;	-- fetch/execute cState bit
	nState : inout std_logic;	-- next cState
	ldIR : out std_logic;	-- ldIR - load control signal
	ldIP : out std_logic;
	ldACC : inout std_logic;
	selMuxA : out std_logic;
	selMuxB : out std_logic;
	oeACC : out std_logic;
	sel_ALUf : out std_logic_vector(2 downto 0)
	);
            
	END COMPONENT;

    SIGNAL reset : std_logic;
    SIGNAL  cstate :  std_logic;    -- fetch/execute state bit
	 signal nstate : std_logic;
    SIGNAL  selMuxA :  std_logic;
    SIGNAL  selMuxB :  std_logic;

    SIGNAL  ldIR :  std_logic;    -- ldIR - load control signal
    SIGNAL  ldIP :  std_logic;
    SIGNAL  ldACC :  std_logic;

    SIGNAL  IR :  std_logic_vector(15 downto 0); -- instruction register
    SIGNAL  IP :  std_logic_vector(15 downto 0); -- program counter

    SIGNAL  ACC :  std_logic_vector(15 downto 0);    -- accumulator

    SIGNAL mio :  std_logic;
    SIGNAL mwe :  std_logic;
 
    SIGNAL  zeroACC :  std_logic;

    signal  oeACC : std_logic;
    SIGNAL  sel_ALUf :  std_logic_vector(2 downto 0);
      
           
BEGIN

	uut: LDA PORT MAP(
		reset => reset,
        selMuxA => selMuxA,
        selMuxB => selMuxB, 
        mio => mio,
        mwe => mwe,
		
        IR => IR,
        IP => IP,
        ACC => ACC,
        zeroACC => zeroACC,
           -- control unit
        cstate => cstate, 
		  nstate => nstate,
        ldIR => ldIR, 
        ldIP => ldIP,
        ldACC => ldACC,
        oeACC => oeACC,
        sel_ALUf => sel_ALUf
	);



	
   tb_LDA : PROCESS
   BEGIN
      reset <= '1';
      wait for 100 ns;
      reset <= '0';
		cState <= '0';
		wait for 100 ns;
		cState <= '1';
		IR <= "0000000000000000";

      wait;
   END PROCESS;

END;
