----------------------------------------------------------------------------------
-- NoC_CPU2S_tb
-- CBP 10-01-23
-- CBP 17-01-23  Cut down version for studs
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
USE ieee.numeric_std.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity NoC_CPU2S_tb is
end NoC_CPU2S_tb;

architecture behavioral of NoC_CPU2S_tb is

component NoC_CPU2S is
  Port ( 
    clk   : in std_logic;
    reset : in std_logic;
    state     : inout std_logic	 ;    
    IP    : inout std_logic_vector(15 downto 0);
    IR    : inout std_logic_vector(15 downto 0);
    -- buses
    aBus  : inout std_logic_vector(15 downto 0);
    dBus1 : inout std_logic_vector(15 downto 0);
    dBus2 : inout std_logic_vector(15 downto 0);  
    ACC   :  inout std_logic_vector(15 downto 0); 
    -- register loads  
  	ldACC : inout std_logic;
  	ldIP  : inout std_logic;
  	ldIR  : inout std_logic;
  	-- mux selections
  	selMuxA : inout std_logic;
  	selMuxB : inout std_logic;
  	selMuxC : inout std_logic;
  	selALUf : inout std_logic_vector(2 downto 0);
  	-- memory studd
  	mio : inout std_logic;
  	mwe : inout std_logic;
  	-- i/o stuff
	inPort    : in std_logic_vector(15 downto 0); 
	outPort   : inout std_logic_vector(15 downto 0); 
    ldOutPort :  inout std_logic;	-- loads output port from data bus 2
	oeInPort  : inout std_logic;		-- enables input port onto data bus 1  	
	sw0       : in std_logic; 	
	ALUout  :  inout std_logic_vector(15 downto 0);
	MUXBout  :  inout std_logic_vector(15 downto 0)
  );
end component;

signal reset : std_logic;
signal clk   : std_logic;
signal state : std_logic;
signal IP : std_logic_vector(15 downto 0);
signal IR : std_logic_vector(15 downto 0);
    -- buses
signal     aBus  : std_logic_vector(15 downto 0);
--signal     MUXBout : std_logic_vector(15 downto 0);  
signal     dBus1 : std_logic_vector(15 downto 0);
signal     dBus2 : std_logic_vector(15 downto 0);  
signal     ACC   :  std_logic_vector(15 downto 0); 

-- To simplify comment out all lines from here to end -----
    -- register loads  
signal   	ldACC : std_logic;
signal   	ldIP  : std_logic;
signal   	ldIR  : std_logic;
--  	 mux selections
signal  	selMuxA : std_logic;
signal   	selMuxB : std_logic;
signal      selMuxC : std_logic;
signal   	selALUf : std_logic_vector(2 downto 0);
  	-- memory studd
signal   	mio : std_logic;
signal   	mwe : std_logic;
-- End of simplification commenting out -----------

  	-- i/o stuff
signal 	inPort    : std_logic_vector(15 downto 0); 
signal 	outPort   : std_logic_vector(15 downto 0); 

-- To simplify comment out next 2 lines ------------
signal  ldOutPort :  std_logic;	-- loads output port from data bus 2
signal  oeInPort  : std_logic;		-- enables input port onto data bus 1  	 

signal sw0 : std_logic;

-- To simplify comment out next 3 lines -------------
signal ALUout : std_logic_vector(15 downto 0);
signal MUXBout : std_logic_vector(15 downto 0);  

begin

uut : NoC_CPU2S port map(
	clk => clk,
	reset => reset,
	IP => IP,
	IR => IR,
	aBus => aBus,
	
	-- To simplify  comment out next line ------
	MUXBout =>  MUXBout,
	dBus1 => dBus1,
	dBus2 => dBus2,
	ACC => ACC,
	
	-- To simplify comment out from here -------
	ldACC => ldACC,
	ldIP => ldIP,
	ldIR => ldIR,
	selMuxA => selMuxA,
	selMuxB => selMuxB,
	selMuxC => selMuxC,
	selALUf => selALUf,
	mio => mio,
	mwe => mwe,
    -- End of simplification comment out -------
    
	inPort => inPort,
	outPort => outPort,
	
	-- To simplify the next 2 lines ------------
	ldOutPort => ldOutPort,
	oeInPort => oeInPort,

	sw0 => sw0,
	state => state,
	
	-- So simplify comment out next line -------
	ALUout => ALUout
);

-- NB can't set same signal in two processes, they will conflict!
tb_clock : process
begin
	clk <= '0';
	wait for 100 ns;
	clk <= '1';
	wait for 100 ns;
end process;

tb_reset : process
begin
	reset <= '1';
	wait for 210 ns;
	reset <= '0';
	wait;
end process;

tb_input : process
begin
    wait for 50 ns;
    --dummy <= '1';
    wait for 50 ns;
    sw0 <= '1';
    wait for 10 ns; -- this wait is really needed !
    inport <= "000000000000000" & sw0;
    wait;
end process;
    

end Behavioral;
