----------------------------------------------------------------------
--  ISE_Control.vhd
--  CBP 06-01-23
--  Mods 16-01-23
-----------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity Control is
    Port ( 
    clk : in std_logic;
    -- inputs to controller
    reset   : in std_logic;
 	IR      : in std_logic_vector(15 downto 0); -- instruction register
 	--cState  : in std_logic; 
	ACC     : in std_logic_vector(15 downto 0);	-- accumulator
	zeroACC : in std_logic;
	 	   
    mio     : out std_logic;
    mwe     : out std_logic;

	ldIR    : out std_logic;	-- ldIR - load control signal
	ldIP    : out std_logic;
	ldACC   : out std_logic;
	selMuxA : out std_logic;
	selMuxB : out std_logic;
	selMuxC : out std_logic;
	selALUf : out std_logic_vector(2 downto 0);
	ldOutPort : out std_logic;
	oeInPort : out std_logic;
    state   : inout std_logic
	);   
end Control;

architecture Behavioral of Control is

type fsm_states is (FETCH,EXEC);
signal  cState, nState : fsm_states;

begin

    ControlUnitState : process(reset,clk)
    begin
    if(reset = '1') then
        cState <= FETCH;
    else
        if (rising_edge(clk)) and reset = '0' then
            cstate <= nState;
        end if;
    end if;
    end process;
        

		
	ControlUnit: process(reset, IR, ACC, zeroACC, cState)
	begin
	if(cState = FETCH) then
	  state <= '0';
	else
	  state <= '1';
	end if;

		-- zero all signals
		mio <= '0';
		mwe <= '0';
		ldIR <= '0';
		ldIP <= '0';
		ldACC <= '0';
		selMuxA <= '0';
		selMuxB <= '0';
		selMuxC <= '0';
		--oeACC <= '0';
		nState <= FETCH;
		selALUf <= "000";
		ldOutPort <= '0';
		oeInPort <= '0';
		
		if (reset = '1') then
			ldIP <= '1';
			ldIR <= '1';
			ldACC <= '1';
			mio <='1';
			nState <= EXEC;  
		else 
			if (cState = FETCH) then 			  	-- FETCH
				mio <= '1'; 
				selALUf <= "010";
				ldIP <= '1';
				ldIR <= '1';
				nState <= EXEC;	
			else									   -- EXECUTE
				case IR(15 downto 12) is
					when "1000" => -- IN 
						mio <= '0';
						selMuxA <= '1';
						selMuxB <= '1';
						selMuxC <= '1';           -- connedt IOPort to dBus1 
						selALUf <= "001";
						ldACC <='1';
						nState <= FETCH;
						oeInPort <= '1';
					when "1001" => -- OUT 
						mio <= '0';
						--oeACC <= '1';
						nState <= FETCH;
						ldOutPort <= '1';
					when "0000" => -- LDA 
						mio <= '1';
						selMuxA <= '1';
						selMuxB <= '1'; 
						selALUf <= "001";
						ldACC <='1';
						nState <= FETCH;
					when "0001" => -- STO
						mio <= '1';
						mwe <= '1';
						--oeACC <= '1';
						selMuxA <= '1';
						oeInport <= '0';
						nState <= FETCH;
					when "0010" => -- ADD S
						mio <= '1';
						selMuxA <= '1';
						selMuxB <= '1';
						selALUf <= "011";
						ldACC <= '1';
						nState <= FETCH;
					when "0011" => -- SUB S
						mio <= '1';
						selMuxA <= '1'; selMuxB <= '1';
						selALUf <= "100";
						ldACC <= '1';
						nState <= FETCH;
					when "0100" => -- JMP S
						mio <= '1';
						selMuxA <= '1'; 
						selALUf <= "010";
						ldIP <= '1';
						ldIR <= '1';
						nState <= EXEC;
					when "0101" => -- JGE S
						if (ACC(15) = '0') then
							mio <= '1';
							selMuxA <= '1'; 
							selALUf <= "010";
							ldIP <= '1';
							ldIR <= '1';
							nState <= EXEC;
						else
							mio <= '1'; 
							selALUf <= "010";
							ldIP <= '1';
							ldIR <= '1';
							nState <= EXEC;
						end if;
					when "0110" => -- JNE S
						if (zeroACC = '0') then
							mio <= '1';
							selMuxA <= '1'; 
							selALUf <= "010";
							ldIP <= '1';
							ldIR <= '1';
							nState <= EXEC;
						else
							mio <= '1'; 
							selALUf <= "010";
							ldIP <= '1';
							ldIR <= '1'; 
							nState <= EXEC;
						end if;
					when "0111" => -- HLT
						-- stop generating signals
		                  mio <= '0';
	                   	  mwe <= '0';
		                  ldIR <= '0';
		                  ldIP <= '0';
		                  ldACC <= '0';
		                  selMuxA <= '0';
		                  selMuxB <= '0';
		                 -- oeACC <= '0';
		                  nState <= EXEC;  -- very inportant
		                  selALUf <= "000";
		                  ldOutPort <= '0';
		                  oeInPort <= '0';
					when others =>
		                  mio <= '0';
		                  mwe <= '0';
		                  ldIR <= '0';
		                  ldIP <= '0';
		                  ldACC <= '0';
		                  selMuxA <= '0';
		                  selMuxB <= '0';
		                  --oeACC <= '0';
		                  nState <= EXEC;
		                  selALUf <= "000";
		                  ldOutPort <= '0';
		                  oeInPort <= '0';					
				end case;
			end if;
		end if;

	end process;

end Behavioral;
