----------------------------------------------------------------------------------
-- NoC_CPU2S.vhd
-- Structural Architecture for CPU2
-- CBP 11-01-23
----------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.std_logic_arith.all;

entity NoC_CPU2S is
  Port ( 
    clk : in std_logic;
    reset : in std_logic;
    IP : inout std_logic_vector(15 downto 0);
    IR : inout std_logic_vector(15 downto 0);
    -- buses
    aBus  : inout std_logic_vector(15 downto 0);
    dBus1 : inout std_logic_vector(15 downto 0);
    dBus2 : inout std_logic_vector(15 downto 0); 
    ACC   :  inout std_logic_vector(15 downto 0); 
    -- register loads  
  	ldACC : inout std_logic;
  	ldIP  : inout std_logic;
  	ldIR  : inout std_logic;
  	-- mux selections
  	selMuxA : inout std_logic;
  	selMuxB : inout std_logic;
  	selMuxC : inout std_logic;  	
  	selALUf : inout std_logic_vector(2 downto 0);
  	-- memory stuff
  	mio : inout std_logic;
  	mwe : inout std_logic;
  	-- i/o stuff
	inPort    : in std_logic_vector(15 downto 0); 
	outPort   : inout std_logic_vector(15 downto 0); 
    ldOutPort :  inout std_logic;	-- loads output port from data bus 2
	oeInPort  : inout std_logic;	-- enables input port onto data bus 1  
	--
	--sw0 : in std_logic;	 
	state : inout std_logic;	 
	ALUout 	:  inout std_logic_vector(15 downto 0);
	MUXBout 	:  inout std_logic_vector(15 downto 0)	
  );
end NoC_CPU2S;

architecture Structural of NoC_CPU2S is


signal busALUout : std_logic_vector(15 downto 0);

signal busMUXBALU : std_logic_vector(15 downto 0);

signal zeroFlag : std_logic;

signal busIPMUXA : std_logic_vector(15 downto 0);
signal busIRMUXA : std_logic_vector(15 downto 0);
--7
signal accOUT     : std_logic_vector(15 downto 0);
signal inPort_out : std_logic_vector(15 downto 0);
signal MEM_out    : std_logic_vector(15 downto 0);
signal MUXA_out   : std_logic_vector(15 downto 0);

component reg is
    Port ( 
    clk      : in std_logic;
    load     : in std_logic;
    data_in  : in std_logic_vector(15 downto 0);
	data_out : out std_logic_vector(15 downto 0) -- ALU output
	);   
end component;

component IOPort is
    Port ( 
    load     : in std_logic;
    data_in  : in std_logic_vector(15 downto 0);
	data_out : out std_logic_vector(15 downto 0) -- ALU output
	);   
end component;

component alu is
    Port ( 
	inA      : in std_logic_vector(15 downto 0);	-- from ACC
	inB      : in std_logic_vector(15 downto 0); 	-- from MUX-B
	sel_ALUf : in std_logic_vector(2 downto 0);	
	op       : out std_logic_vector(15 downto 0);	-- ALU output
	zero     : out std_logic
	);		   
end component;

component mux is
   Port ( 
	inA : in std_logic_vector(15 downto 0); 
	inB : in std_logic_vector(15 downto 0); 
	op  : inout std_logic_vector(15 downto 0); -- Bmux output
	sel : in std_logic
	);	   
end component;

component Control is
    Port ( 
    clk : in std_logic;
    reset   : in std_logic;
	state : inout std_logic;    
 	IR      : in std_logic_vector(15 downto 0); -- instruction register
	ACC     : in std_logic_vector(15 downto 0);	-- accumulator
	zeroACC : in std_logic;	 	   
    mio     : out std_logic;
    mwe     : out std_logic;
	ldIR    : out std_logic;	-- ldIR - load control signal
	ldIP    : out std_logic;
	ldACC   : out std_logic;
	selMuxA : out std_logic;
	selMuxB : out std_logic;
	selMuxC : out std_logic;
	selALUf : out std_logic_vector(2 downto 0);
	ldOutPort : out std_logic;
	oeInPort : out std_logic
	);   
end component;

component RAM is
    Port ( 
    clk      : in std_logic;
    addr_in  : in std_logic_vector(15 downto 0);
    data_in  : in std_logic_vector(15 downto 0);    
    data_out : out std_logic_vector(15 downto 0);
    mio      : in std_logic;
    mwe      : in std_logic
	);		   
end component;

begin

compACC: reg port map(clk,ldACC,busALUout,accOUT);
dBus2 <= accOUT;
ACC <= accOUT;  -- just for testbench

compALU: alu port map(dBus2,busMUXBALU,selALUf,busALUout,zeroFlag);

compIP : reg port map(clk,ldIP,busALUout,busIPMUXA);

compIR : reg port map(clk,ldIR,dBus1,busIRMUXA);

compMUXA : mux port map(busIRMUXA,busIPMUXA,MUXA_Out,selMuxA);
aBus <= "00000000000" & MUXA_Out(4 downto 0);
--
compMUXB : mux port map(dBus1,aBus,busMUXBALU,selMuxB);

compOUTP : reg port map(clk,ldOutPort,dBus2,outPort);

compINP : IOPort port map(oeInPort,inPort,inPort_Out);

compRAM : RAM port map(clk,aBus,dBus2,mem_Out,mio,mwe);

compMUXC : mux port map(inPort_Out,mem_Out,dBus1,selMUXC);

compCtrl : Control port map(clk,reset,state,busIRMUXA,busALUout,zeroFlag,
            mio,mwe,ldIR,ldIP,ldACC,selMUXA,selMUXB,selMUXC,selALUf,ldOutPort,oeInPort);

-- following to get IR and IP onto testbench
IP <= busIPMUXA;
IR <= busIRMUXA;

ALUout <= busALUout;   -- just for testbench
MUXBout <= busMUXBALU; -- for tb

end Structural;
