----------------------------------------------------------------------------------
-- pin_lock_tb.vhd
-- testbench for pin_lock.vhd
-- CBP 23-12-22
----------------------------------------------------------------------------------

  LIBRARY ieee;
  USE ieee.std_logic_1164.ALL;
  USE ieee.numeric_std.ALL;

  ENTITY pin_lock_tb IS
  END pin_lock_tb;

  ARCHITECTURE behavioral OF pin_lock_tb IS 

  COMPONENT pin_lock
  port( clock : in std_logic;  
        button_reset : in std_logic;   
        button_1 : in std_logic;    
        button_2 : in std_logic;    
        button_3 : in std_logic;    
        button_4 : in std_logic;                    
        unlock_out : out std_logic  
        );
  END COMPONENT;

	signal clock : std_logic := '0';
	signal button_reset : std_logic := '0';
    signal button_1 : std_logic := '0';
    signal button_2 : std_logic := '0';
    signal button_3 : std_logic := '0';
    signal button_4 : std_logic := '0';            
    signal unlock_out : std_logic;
    
    constant clock_period : time := 20 ns;
       
  BEGIN

  -- Component Instantiation --
        uut: pin_lock PORT MAP(
			clock => clock,
			button_reset => button_reset,
			button_1 => button_1,
			button_2 => button_2,
			button_3 => button_3,
			button_4 => button_4,						
			unlock_out => unlock_out
          );

  -- Clock process definitions --
	clock_process : process
	begin
	clock <= '0';
	wait for clock_period/2;
	clock <= '1';
	wait for clock_period/2;
	end process;
	
  --  Stimulus process --
    stim_process : PROCESS
     BEGIN
     -- assert reset --

	 button_reset <= '1';
	 wait for 40 ns;
	 button_reset <= '0';
	 wait for 40 ns;
	 
	 -- start with incorrect sequence --
	 button_1 <= '1';
	 wait for 40 ns;
	 button_1 <= '0';
	 wait for 40 ns;
	 button_2 <= '1';
	 wait for 40 ns;
	 button_2 <= '0';
	 wait for 40 ns;	 
	 button_3 <= '1';
	 wait for 40 ns;
	 button_3 <= '0';
	 wait for 40 ns;	 
	 button_4 <= '1';
	 wait for 40 ns;
     button_4 <= '0';
	 wait for 40 ns;
	  
	 button_reset <= '1';
	 wait for 40 ns;
	 button_reset <= '0';
	 wait for 40 ns;
	 
	 -- now another incorrect sequence --
	 button_1 <= '1';
	 wait for 40 ns;
	 button_1 <= '0';
	 wait for 40 ns;	 
	 button_3 <= '1';
	 wait for 40 ns;
	 button_3 <= '0';
	 wait for 40 ns;	 
	 button_4 <= '1';
	 wait for 40 ns;
	 button_4 <= '0';
	 wait for 40 ns;	 
	 button_2 <= '1';
	 wait for 40 ns;
	 button_2 <= '0';
	 wait for 40 ns;
	 	 
	 button_reset <= '1';
	 wait for 40 ns;
	 button_reset <= '0';
	 wait for 80 ns;
	 	 
--	  now give the correct sequence --
	 button_3 <= '1';
	 wait for 40 ns;
	 button_3 <= '0';
	 wait for 40 ns;	 
	 button_1 <= '1';
	 wait for 40 ns;	 
	 button_1 <= '0';
	 wait for 40 ns;
	 button_2 <= '1';
	 wait for 40 ns;
	 button_2 <= '0';
	 wait for 40 ns;	 
	 button_4 <= '1';
	 wait for 40 ns;	 
	 button_4 <= '0';
	 wait for 100 ns;
	 						
     END PROCESS stim_process;
  
END	behavioral;
